#!/bin/python3

import argparse
import logging
import os
import sys

import architrice.actions as actions
import architrice.utils as utils

APP_NAME = "architrice"

DESCRIPTION = f"""
Download Archidekt decks to a local directory. To set output path:
{APP_NAME} -p OUTPUT_DIRECTORY
This is cached and will be used until a different path is set.

To download a single deck to the set output path:
{APP_NAME} -d ARCHIDEKT_DECK_ID
This deck id is cached, and if the command is run again without argument, the
same deck will be downloaded.

To download all decks for a specific user name:
{APP_NAME} -u ARCHIDEKT_USERNAME
This username is cached, and if the command is run again without argument, the
same user's decks will be downloaded.

To download the most recently updated deck for a specific user:
{APP_NAME} -l
If no user has been set, the user will need to be specified as well through -u.
"""


def parse_args():
    parser = argparse.ArgumentParser(
        description=DESCRIPTION,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-d", "--deck", dest="deck", help="set deck id to download"
    )
    parser.add_argument(
        "-u", "--user", dest="user", help="set username to download decks of"
    )
    parser.add_argument(
        "-p", "--path", dest="path", help="set deck file output directory"
    )
    parser.add_argument(
        "-l",
        "--latest",
        dest="latest",
        action="store_true",
        help="download latest deck for user",
    )
    parser.add_argument(
        "-v",
        "--verbosity",
        dest="verbosity",
        action="count",
        help="increase output verbosity",
    )
    parser.add_argument(
        "-q",
        "--quiet",
        dest="quiet",
        action="store_true",
        help="disable log output",
    )
    return parser.parse_args()


def main():
    args = parse_args()

    utils.set_up_logger(
        0 if args.quiet else args.verbosity + 1 if args.verbosity else 1
    )

    if len(sys.argv) == 1 and not utils.cache_exists():
        cache = actions.setup_wizard()
    else:
        cache = utils.load_cache()

    user = args.user or cache["user"]
    deck = args.deck or cache["deck"]
    path = utils.expand_path(args.path) if args.path else cache["path"]

    if os.path.isfile(path):
        logging.error(
            f"Fatal: Output directory {path} already exists and is a file."
        )
        exit()

    if not os.path.isdir(path):
        os.makedirs(path)
        logging.info(f"Created output directory {path}.")

    cache.update({"user": user, "deck": deck, "path": path})

    if cache["dirs"].get(path) is None:
        cache["dirs"][path] = dir_cache = {}
    else:
        dir_cache = cache["dirs"][path]

    if path is None:
        print(
            f"No output file specified. Set one with {APP_NAME} -p"
            " OUTPUT_DIRECTORY."
        )
    elif args.latest:
        if not user:
            print(
                f"No Archidekt user set. Set one with {APP_NAME} -u"
                " ARCHIDEKT_USERNAME to download their latest deck."
            )
        else:
            print(f"Downloading latest deck for Archidekt user {user}.")
            actions.download_latest(user, path, dir_cache)
    elif user:
        print(f"Updating all decks for Archidekt user {user}.")
        actions.download_all(user, path, dir_cache)
    elif deck:
        print(f"Updating deck with Archidekt id {deck}.")
        actions.download_deck(deck, path, dir_cache)
    elif args.path:
        print(f'Set output directory to "{path}".')
    else:
        print("No action specified. Nothing to do.")

    utils.save_cache(cache)


if __name__ == "__main__":
    main()



import requests

import architrice.utils as utils

URL_BASE = "https://archidekt.com/api/decks/"

# Note that the api is sensitive to double slashes so /api/decks//id for
# instance will fail.

SIDEBOARD_CATEGORIES = {"Commander", "Maybeboard", "Sideboard"}


def deck_to_generic_format(deck):
    main = []
    side = []

    for card in deck["cards"]:
        card_tuple = (
            card["quantity"],
            card["card"]["oracleCard"]["name"],
            "dfc" in card["card"]["oracleCard"]["layout"],
        )
        if belongs_in_sideboard(card["categories"]):
            side.append(card_tuple)
        else:
            main.append(card_tuple)

    return {
        "name": deck["name"],
        "description": deck["description"],
        "main": main,
        "side": side,
    }


def get_deck(deck_id, small=True):
    return deck_to_generic_format(
        requests.get(
            URL_BASE + f"{deck_id}{'/small' if small else ''}/",
            params={"format": "json"},
        ).json()
    )


def deck_list_to_generic_format(decks):
    ret = []
    for deck in decks:
        ret.append(
            {
                "id": str(deck["id"]),
                "updated": utils.parse_iso_8601(deck["updatedAt"]),
            }
        )
    return ret


def get_deck_list(user_name):
    decks = []
    url = URL_BASE + f"cards/?owner={user_name}&ownerexact=true"
    while url:
        j = requests.get(url).json()
        decks.extend(j["results"])
        url = j["next"]

    return deck_list_to_generic_format(decks)


def belongs_in_sideboard(categories):
    return bool(SIDEBOARD_CATEGORIES.intersection(categories))



import xml.etree.cElementTree as et

import architrice.utils as utils

COCKATRICE_DECK_FILE_EXTENSION = ".cod"


def cockatrice_name(card_tuple):
    # Cockatrice implements dfcs as a seperate card each for the front and
    # back face. By adding just the front face, the right card will be in the
    # deck.
    _, name, is_dfc = card_tuple
    if is_dfc:
        return name.split("//")[0].strip()
    return name


def deck_to_xml(deck, outfile):
    root = et.Element("cockatrice_deck", version="1")

    et.SubElement(root, "deckname").text = deck["name"]
    et.SubElement(root, "comments").text = deck["description"]

    main = et.SubElement(root, "zone", name="main")
    side = et.SubElement(root, "zone", name="side")

    for card in deck["main"]:
        et.SubElement(
            main,
            "card",
            number=str(card[0]),
            name=cockatrice_name(card),
        )
    for card in deck["side"]:
        et.SubElement(
            side, "card", number=str(card[0]), name=cockatrice_name(card)
        )

    et.ElementTree(root).write(outfile, xml_declaration=True, encoding="UTF-8")


def save_deck(deck, path):
    deck_to_xml(deck, path)


def create_file_name(deck_name):
    return utils.create_file_name(deck_name) + COCKATRICE_DECK_FILE_EXTENSION



import setuptools

setuptools.setup(
    name="architrice",
    version="0.0.1",
    url="https://github.com/OwenFeik/architrice.git",
    author="Owen Feik",
    author_email="owen.h.feik@gmail.com",
    description="Utility to download Archidekt decklists for local use.",
    packages=setuptools.find_packages(),
    download_url="https://github.com/OwenFeik/architrice/archive/refs/tags/0.0.1.tar.gz",
    install_requires=["requests"]
)



import datetime
import json
import logging
import os
import re
import sys

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
CACHE_FILE = os.path.join(DATA_DIR, "cache.json")
LOG_FILE = os.path.join(DATA_DIR, "log")

DEFAULT_CACHE = {"user": None, "deck": None, "path": None, "dirs": {}}


def set_up_logger(verbosity=1):
    handlers = [logging.FileHandler(LOG_FILE)]

    if verbosity != 0:
        stdout_handler = logging.StreamHandler(sys.stdout)
        stdout_handler.setFormatter(
            logging.Formatter("%(levelname)s: %(message)s")
        )
        stdout_handler.setLevel(
            logging.INFO if verbosity == 1 else logging.DEBUG
        )
        handlers.append(stdout_handler)

    logging.basicConfig(level=logging.DEBUG, handlers=handlers)


# Cache format:
#   {
#       "user": "archidekt username",
#       "deck": "last deck downloaded",
#       "path": "output directory",
#       "dirs": {
#           "PATH_TO_DIR": {
#               "ARCHIDEKT_DECK_ID": {
#                   "updated": timestamp
#                   "name": "file name"
#               } ... for each deck downloaded
#           } ... for each deck directory
#       }
#   }
def load_cache():
    if os.path.isfile(CACHE_FILE):
        with open(CACHE_FILE, "r") as f:
            return json.load(f)
    else:
        return DEFAULT_CACHE


def save_cache(cache):
    if not os.path.isdir(DATA_DIR):
        os.mkdir(DATA_DIR)

    with open(CACHE_FILE, "w") as f:
        json.dump(cache, f, indent=4)


def cache_exists():
    return os.path.isfile(CACHE_FILE)


def create_file_name(deck_name):
    return re.sub("[^a-z0-9_ ]+", "", deck_name.lower()).replace(" ", "_")


def parse_iso_8601(time_string):
    return datetime.datetime.strptime(
        time_string, "%Y-%m-%dT%H:%M:%S.%fZ"
    ).timestamp()


def expand_path(path):
    return os.path.abspath(os.path.expanduser(path))



import asyncio
import concurrent.futures
import datetime
import logging
import os

import architrice.archidekt as archidekt
import architrice.cockatrice as cockatrice
import architrice.utils as utils

source = archidekt
target = cockatrice

# The reason for this "source" and "target" setup is that it allows future
# drop in replacements for other sources (different deck hosting websites)
# and targets (different mtg clients).
#
# To facilitate this, generic intermediate formats for decks and lists
# thereof are defined as follows.
#
# Generic deck format:
#   {
#       "name": "Deck Title",
#       "file_name": "deck_title.file",
#       "description": "Description of deck",
#       "main": [
#           (quantity, card_name, is_dfc) ... for card in main deck
#       ],
#       "side": [
#           (quantity, card_name, is_dfc) ... for card in sideboard
#       ]
#   }
#
# Generic list of decks format:
#   [
#       {
#           "id": "ARCHIDEKT_DECK_ID",
#           "updated": UTC_TIMESTAMP
#       } ... for each deck
#   ]

THREAD_POOL_MAX_WORKERS = 12


def download_deck(deck_id, path, dir_cache):
    if deck_id in dir_cache:
        logging.debug(f"Updating existing deck {deck_id}.")
        deck_cache = dir_cache[deck_id]
    else:
        logging.debug(f"Downloading new deck {deck_id}.")
        dir_cache[deck_id] = deck_cache = {"name": None, "updated": 0}

    deck = source.get_deck(deck_id)

    if deck_cache["name"]:
        deck["file_name"] = deck_cache["name"]
    else:
        deck_cache["name"] = deck["file_name"] = target.create_file_name(
            deck["name"]
        )

    target.save_deck(deck, os.path.join(path, deck["file_name"]))
    deck_cache["updated"] = datetime.datetime.utcnow().timestamp()
    logging.info(f"Successfully downloaded {deck['name']} ({deck_id}).")


def decks_to_update(username, dir_cache):
    decks = source.get_deck_list(username)
    logging.info(f"Total decks: {len(decks)}.")

    to_download = []
    for deck in decks:
        if (
            deck["id"] not in dir_cache
            or deck["updated"] > dir_cache[deck["id"]]["updated"]
        ):
            to_download.append(deck["id"])

    logging.info(f"To update: {len(to_download)}.")

    return to_download


def download_latest(username, path, dir_cache):
    to_download = decks_to_update(username, dir_cache)
    if len(to_download) == 0:
        logging.info("All decks up to date.")
    else:
        download_deck(
            max(to_download, key=lambda d: d["updated"])["id"], path, dir_cache
        )


# This is asynchronous so that it can use a ThreadPoolExecutor to speed up
# perfoming many deck requests.
async def download_decks_pool(loop, decks, path, dir_cache):
    with concurrent.futures.ThreadPoolExecutor(
        max_workers=THREAD_POOL_MAX_WORKERS
    ) as executor:
        futures = [
            loop.run_in_executor(
                executor, download_deck, deck_id, path, dir_cache
            )
            for deck_id in decks
        ]
        return await asyncio.gather(*futures)


def download_all(username, path, dir_cache):
    logging.info(f"Updating all decks for {username}.")

    loop = asyncio.get_event_loop()
    loop.run_until_complete(
        download_decks_pool(
            loop, decks_to_update(username, dir_cache), path, dir_cache
        )
    )

    logging.info(f"Successfully updated all decks for {username}.")


def setup_wizard():
    cache = utils.DEFAULT_CACHE
    cache["user"] = input("Archidekt username > ")
    cache["path"] = utils.expand_path(input("Output directory > "))
    return cache
